#! /usr/bin/env python

try:
	import string, sys, os, pci
	from device import *
	from struct import pack, unpack
	from array import array
	from monitor import *
	from edid import Edid
	from pci_id import *
	from fb import Framebuffer			
	### Only load the vbe module on x86
	arch = os.uname()[4]
	if arch.count("i386") or arch.count("i486") or arch.count("i586") or arch.count("i686"):
		from vbe import *

except ImportError, e:
	print "Module loading failed on: " + str(e)
	print "Please check that xac was installed correctly"
	sys.exit()

### Corresponds to the video card class (mask 0x0300)
PCI_BASE_CLASS_DISPLAY = 3
### We only want the upper byte, PCI is in LE
PCI_DEVICE_CLASS_HIGH = 11

### Arch specific routines
class Arch:
	def __init__(self, binary = 0):
		### Arch name
		self.arch_name = "None"
		### Do we support binary drivers?
		self.binary_drivers = binary
		### PCI reference, so we don't keep making new ones
		self.p = pci.PCI()
		self.pci_devs = self.p.list_pci_devs()
		self.pci_cards = []
		### Ignore the FB device when configuring the monitor
		self.ignore_fb = 0
		### Ignore the EDID when configuring the monitor
		self.ignore_edid = 0
		### Framebuffer device
		self.fb = Framebuffer()
		### Print the warning about not finding the video card?
		self.warning = 0
		### PCI Video Cards Found
		for card in self.pci_devs:
			info = self.p.get_config_info(card[0], card[1], card[2])
			if info and unpack('B',info[PCI_DEVICE_CLASS_HIGH])[0] == PCI_BASE_CLASS_DISPLAY:
				self.pci_cards.append(card)
		### Fill the video card list
		self.cards = []
		self.get_video_cards()

	### Enable / disable binary drivers
	def set_binary_drivers(self, binary):
		if not self.binary_drivers == binary:
			self.binary_drivers = binary
			self.cards = []
			self.get_video_cards()
		
	### Override this for each arch
	def get_edids(self):
		return None
	
	### Gets the machine type for machine specific workarounds
	def get_machine(self):
		return os.uname()[4]

	### Default card that's picked if nothing is selected
	def arch_default_driver(self):
		return Dev_fbdev()

	### Gets a list of available video cards (returns XAC_Card refs)
	def get_video_cards(self):
		cards = []
		
		### If we can tell the card type from the FB, use that as our
		### default (1st in array) card
		d = self.fb.fb_id_to_driver(self.binary_drivers)
		if d:
			### Remove the FB card from the PCI list
			### We guess that it's the first card of the FB type
			### So if it's radeon, remove the first radeon card...
			### Is there a better way to match it?
			for card in self.pci_cards:
				info = self.p.get_config_info(card[0], card[1], card[2])
				vendor = unpack('<H', info[0:2])[0]
				device = unpack('<H', info[2:4])[0]
				test = pci_to_driver(vendor, device, self.binary_drivers)
				if test and test.driver == d.driver:
					c = XAC_Card(d, card, len(cards), self.fb.fb_string())
					c.compatible_drivers = pci_to_compatible(vendor, device, self.binary_drivers, self.arch_name)
					### Add the FB card to the cards array
					cards.append(c)
					### Remove the FB card from the PCI List
					self.pci_cards.remove(card)
					break
			### No PCI Device exists for this card, add it
			else:
				c = XAC_Card(d, None, len(cards), self.fb.fb_string())
				### Assume that compatible cards were already set
				c.compatible_drivers = self.fb.fb_id_to_compatible(self.binary_drivers)	
				### Add the FB card to the cards array
				cards.append(c)

		### Try to find additional cards from the PCI list
		for card in self.pci_cards:
			info = self.p.get_config_info(card[0], card[1], card[2])
			vendor = unpack('<H', info[0:2])[0]
			device = unpack('<H', info[2:4])[0]
			d = pci_to_driver(vendor, device, self.binary_drivers)
			if d:
				### Create a card ref and add it to the cards list
				c = XAC_Card(d, card, len(cards), d.driver_description) 	
				c.compatible_drivers = pci_to_compatible(vendor, device, self.binary_drivers, self.arch_name)
				cards.append(c)

		### We didn't find a card.  Use the arch default
		if not len(cards) > 0:
			self.warning = 1
			d = self.arch_default_driver()
			c = XAC_Card(d, None, 0, "Unknown - Architecture Default")
			c.monitors.append(XAC_Monitor())
			cards.append(c)

		### Fill in the monitors that we can find.  Assume that the 
		### found order is the same as the device order
		edids = self.get_edids()
		card = 0
		for e in edids:
			ed = Edid(e)	
			if ed.valid:
				cards[card].monitors.append(XAC_Monitor(ed))
				card = card + 1	

		### If the primary card doesn't have a monitor and we have a FB
		### use the FB to fill in the monitor info
		if not len(cards[0].monitors):
			cards[0].monitors.append(XAC_Monitor(fb=self.fb))	

		### use the default if we don't have a monitor
		for c in cards:
			if not len(c.monitors):
				### Default monitor
				c.monitors.append(XAC_Monitor())	
	
		### Fill the cards ref
		self.cards = cards

	### FIXME Dump more useful debugging info
	def dump_info(self):
		print 'Machine Name:', self.get_machine()
		### Collected Card Information
		print 'Video Cards Found:' 
		for c in self.cards:
			print ' * ' + c.name + ' - (' + c.get_pci_loc() + ')'
			print '\tDetected as: ' + c.driver.driver_description

		### Framebuffer info
		if self.fb.fix and self.fb.var:
			print '\nFramebuffer Info:'
			print '\tAccel ID: ' + str(self.fb.fix['accel'])
			print '\tName String: ' + self.fb.fb_string()
			print '\tDepth PP: ' + str(self.fb.var['bits_per_pixel'])

		### EDID
		edidid = 0
		for ed in self.get_edids():
			e = Edid(ed)			
			if not e.valid:
				continue
			print '\nEDID' + str(edidid) + ':'
			print "\tID: ",e.get_id_string()
			print "\tSN: ",e.get_serial_number()
			print "\tDT: ",e.get_date()
			print "\tEV: ",e.get_edid_ver()
			print "\tVI: ",e.get_video_input_def()
			print "\tSZ: ",e.get_size()
			print "\tGA: ",e.get_gamma()
			print "\tFS: ",e.get_feature_support()
			print "\tCC: ",e.get_color_characteristics()
			print "\tET: ",e.get_timings()
			print "\tMD: ",e.get_monitor_details()
			print "\tHE: ",e.has_extension()
			edidid = edidid + 1

		if edidid == 0:
			print '\nNo EDID information available\n'
		return

	### Arch specific workarounds
	def arch_workarounds(self):
		return

### PowerPC Specific functions
class Arch_ppc(Arch):
	def __init__(self):
		Arch.__init__(self)
		self.arch_name = "ppc"

	### Get edid monitor capabilities via OF
	def get_edids(self):
		found_edid_path = []
		e = []

		### Find the EDID nodes
		for root, dirs, files in os.walk("/proc/device-tree"):
			for f in files:
				### Put the primary display EDID first in the list
				if f == "EDID" and files.count("linux,boot-display"):
					found_edid_path = [root + "/" + f] + found_edid_path
				elif f == "EDID":
					found_edid_path = found_edid_path + [root + "/" + f] 
					
		### For each node, create an edid ref and read it in
		for node in found_edid_path:
			try:
				edid_file = open(node, mode="rb")
				### EDID is 128 bytes
				e = e + [edid_file.read(128)]
				edid_file.close()
			except:
				print "### Unable to read from edid: " + node
		return e
	
	### Gets the machine type for machine specific workarounds
	def get_machine(self):
		cpuinfo = open("/proc/cpuinfo", mode="r")	
		for line in cpuinfo.readlines():
			if line[:7] == "machine" :
				return string.split(line)[2]
		return ""

	### Do ppc specific workarounds
	def arch_workarounds(self):
		m = self.get_machine()
	
		### Force the VertMin on 4,2 (From Xorgautoconfig)
		if m.count("PowerMac4,2"):
			for d in self.cards:
				for mon in d.monitors:
					mon.sync['h_min'] = 28
					mon.sync['h_max'] = 49
					mon.sync['v_min'] = 50
					mon.sync['v_max'] = 60

		### Force the PCI bus on Peg Radeon Cards
		if m.count("Pegasos"):
			for d in self.cards:
				if d.driver.driver == "radeon":
					d.driver.driver.force_pci = 1

		### Force off shadowfb on PS3
		if m.count("PS3PF"):
			for d in self.cards:
				d.driver.force_shadowfb_off = 1

		### Force 24 bit on G5 PowerMacs
		if m.count("PowerMac11"):
			for d in self.cards:
				for mon in d.monitors:
					mon.default_depth = 24
								
### X86/AMD64 Specific instructions
class Arch_x86(Arch):
	def __init__(self):
		Arch.__init__(self, 1)
		self.arch_name = "x86"

	### Get the edid 
	def get_edids(self):
		found_edid_path = []
		e = []
		### We prefer to get the EDID from ACPI if it's available
		if os.path.exists('/proc/acpi'):
			for root, dirs, files in os.walk("/proc/acpi"):
				for f in files:
					### Put the primary display EDID first in the list (OF only?)
					if f == "EDID" and files.count("linux,boot-display"):
						found_edid_path = [root + "/" + f] + found_edid_path
					elif f == "EDID":
						found_edid_path = found_edid_path + [root + "/" + f] 
						
			### For each node, create an edid ref and read it in
			for node in found_edid_path:
				try:
					edid_file = open(node, mode="rb")
					### EDID is 128 bytes
					e = e + [edid_file.read(128)]
					edid_file.close()
				except:
					print "### Unable to read from edid: " + node

		### Otherwise, we have to use VESA extensions (x86 asm, BARF)
		### We can only get the primary edid this way
		### See if we can load the VBE extension	
		if len(e) < 1:
			try:
				if len(e) == 0 and vbe_get_edid_supported():
					e = e + [ vbe_get_edid() ]	
			except ImportError, e: 
				print "### Unable to use VBE to read edid"
		return e

	### Override the arch default card
	def arch_default_driver(self):
		return Dev_vesa()

### Alpha
class Arch_alpha(Arch):
	def __init__(self):
		Arch.__init__(self)
		self.arch_name = "alpha"
	
### Sparc
class Arch_sparc(Arch):
	def __init__(self):
		Arch.__init__(self)
		self.arch_name = "sparc"

### MIPS
class Arch_mips(Arch):
	def __init__(self):
		Arch.__init__(self)
		self.arch_name = "mips"

### HPPA
class Arch_hppa(Arch):
	def __init__(self):
		Arch.__init__(self)
		self.arch_name = "hppa"

### Select arch
def get_arch():
	arch = os.uname()[4]
	if arch.count("alpha"):
		return Arch_alpha()
	elif arch.count("hppa"):
		return Arch_hppa()
	elif arch.count("mips"):
		return Arch_mips()
	elif arch.count("sparc"):
		return Arch_sparc()
	### PPC and PPC64
	elif arch.count("ppc"):
		return Arch_ppc()
	else:
		return Arch_x86()
