#! /usr/bin/env python

from template import Section

### Video Card class
### Generic "holder" so we can switch drivers, but keep PCI info
### dev 	Used for Card identification
### pci_loc	Location on PCI Bus (bus, dev, func)
### name	Name used when printing the card
class XAC_Card:
	def __init__(self, driver=None, pci_loc=None, id=0, name=None):
		### PCI Device String
		self.pci_loc = pci_loc
		### Device #
		self.id = id
		### Device Name
		if name:
			self.name = name
		else:
			self.name = "Unknown"
		### Driver
		self.driver = driver
		### Compatible Drivers
		self.compatible_drivers = []
		### Monitors attached to this card
		self.monitors = []

	def get_section(self):
		s = Section()
		s.name("Device")
		s.item("Option\t\t")
		s.add('Identifier\t"Card' + str(self.id) + '"')
		s.add('Driver\t\t"' + self.driver.driver + '"')
		if self.pci_loc:
			s.add('Configured ' + self.name + ' at PCI Bus ID:', 3)
			s.add('BusID\t\t"PCI:' + self.get_pci_loc() + '"', 1)
		s.add_blank()
		s.add('Set driver options:', 3)
		self.driver.set_driver_options(s)
		s.add_blank()
		s.add('Unchanged, default driver options:', 3)
		self.driver.unset_driver_options(s)
		s.add_blank()
		self.driver.options_footer(s)
		return s.get_section()	
	
	def get_pci_loc(self):
		if self.pci_loc == None:
			return "None"
		else:
			return str(self.pci_loc[0]) + ":" + str(self.pci_loc[1]) + ":" + str(self.pci_loc[2])

### Driver class
### Hold driver specific info for each card
class XAC_Device:
	def __init__(self):
		### Driver name
		self.driver = "None"
		### Driver description
		self.driver_description = "None"
		### Does this device support DRI?
		self.dri = 0
		### Can this device support multihead?
		self.multihead = 0
		### How should we setup for multihead?
		self.multihead_type = "None"
		self.multihead_options = ["None"]

		### Quirks
		### Force the device to PCI instead of AGP (Used on PegII)
		self.force_pci = 0
		### Force shadowfb off (PS3)
		self.force_shadowfb_off = 0

	def set_driver_options(self, s):
		s.add('No options set!', 3)
	
	def unset_driver_options(self, s):
		s.add('No options unset!', 3)

	def options_footer(self, s):
		if self.driver != "None":
			s.add('See "man ' + self.driver + '" for more information and options', 3)
	
### Framebuffer driver
class Dev_fbdev(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "fbdev"
		self.driver_description = "Framebuffer Device"

	def set_driver_options(self, s):
		if self.force_shadowfb_off:
			s.addi('"ShadowFB"\t"false"\t# ShadowFB forced off by configuration')

	def unset_driver_options(self, s):
		s.addi('"Rotate"\t"off"\t# Rotates the display', 1)
		if not self.force_shadowfb_off:
			s.addi('"ShadowFB"\t"true"\t# Use the ShadowFB (required for rotation)',1)

### PS3 Framebuffer driver
class Dev_ps3(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "fbdev"
		self.driver_description = "PS3 Framebuffer Device"
		self.force_shadowfb_off = 1

	def set_driver_options(self, s):
		s.addi('"ShadowFB"\t"false"\t# ShadowFB forced off by configuration')

### Radeon with open driver
class Dev_radeon(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "radeon"
		self.driver_description = "ATI Radeon Open Driver"
		self.dri = 1
		self.multihead = 1
		self.multihead_options = ["None", "MergedFB", "Xinerama"]
	
	def set_driver_options(self, s):
		s.addi('"EnablePageFlip"\t"true"\t# Increases Performance')
		if self.multihead and self.multihead_type == "MergedFB":
			s.addi('"MergedFB"\t\t"true"\t# Set to false if you only have one monitor')
		else:
			s.addi('"MergedFB"\t\t"false"\t# Set to true if you want mirrored monitor output')
		### Force PCI if required
		if self.force_pci:
			s.addi('"BusType"\t\t"PCI"')
	
	def unset_driver_options(self, s):
		s.addi('"AccelMethod"\t"XAA"\t# XAA or EXA, but EXA is unstable"',1)
		s.addi('"AGPMode"\t"1"\t# Set to the AGP bus speed, 1 = 1x',1)
		s.addi('"AGPFastWrite"\t"false"\t# Can be unstable',1)
		s.addi('"BackingStore"\t"false"\t# Currently broken',1)
		s.addi('"BIOSHotkeys"\t"false"\t# Enable this to switch outputs with a hotkey',1)
		if not self.force_pci:
			s.addi('"BusType"\t"auto"\t# Force Video Bus to PCI, AGP or PCIE',1)
		s.addi('"MonitorLayout"\t"auto"\t# Set to TMDS for some Apple Flat Panels',1)
		s.addi('"NoAccel"\t"false"\t# Disables ALL hardware acceleration',1)
		s.addi('"RenderAccel"\t"true"\t# Hardware Render Acceleration"',1)
		s.addi('"ReverseDDC"\t"false"\t# Some G5s require this',1)
		s.addi('"SWcursor"\t"false"\t',1)
		s.addi('"UseFBDev"\t"false"\t# Try enabling this if it does not work',1)

### Radeon with binary driver
class Dev_radeonb(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "fglrx"
		self.driver_description = "ATI Radeon Binary Driver"
	
	def set_driver_options(self, s):
		s.addi('"EnablePrivateBackZ"\t"true"')
		s.addi('"UseFastTLS"\t\t"2"')

	def unset_driver_options(self, s):
		s.addi('"ForceMonitors"\t"notv"')
	
### Rage128 with open driver
class Dev_r128(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "r128"
		self.dri = 1
		self.driver_description = "ATI Rage 128"
	
	def set_driver_options(self, s):
		s.addi('"DMAForXv\t"true"\t# Uses DMA for smoother playback, but could be unstable')

	def unset_driver_options(self, s):
		s.addi('"Display"\t"FP"\t# Set the display type: FP, CRT or Mirror',1)
		s.addi('"NoAccel"\t"false"\t# Disables ALL hardware acceleration',1)
		s.addi('"SWcursor"\t"false"\t',1)
		s.addi('"UseFBDev"\t"true"\t# Try disabling this if it does not work',1)

### Mach64 with open driver
class Dev_mach64(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "mach64"
		self.dri = 1
		self.driver_description = "ATI Mach 64"

### nVidia card with open driver
class Dev_nvidia(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "nv"
		self.driver_description = "NVidia RIVA/TNT/GeForce Open Driver"

	def unset_driver_options(self, s):
		s.addi('"CrtcNumber"\t"auto"\t# Select a head to output to, 0 or 1',1)
		s.addi('"NoAccel"\t"false"\t# Disables ALL hardware acceleration',1)
		s.addi('"Rotate"\t"off"\t# Rotates the display', 1)
		s.addi('"ShadowFB"\t"false"\t# Use the ShadowFB (required for rotation)',1)
		s.addi('"UseFBDev"\t"false"\t# Try enabling this if it does not work',1)

### nVidia card with closed driver
class Dev_nvidiab(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "nvidia"
		self.driver_description = "NVidia RIVA/TNT/GeForce Binary Driver"

	def unset_driver_options(self, s):
		s.addi('"NoLogo"\t"false"\t#Disable the logo on startup',1)

### 3dfx card, Voodoo 3 or newer with open driver
class Dev_tdfx(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "tdfx"
		self.dri = 1
		self.driver_description = "Voodoo 3/4/5"

	def unset_driver_options(self, s):
		s.addi('"DRI"\t"true"\t# Enables/disables DRI support', 1)
		s.addi('"NoAccel"\t"false"\t# Disables ALL hardware acceleration',1)
		s.addi('"TexturedVideo"\t"false"\t# Use the texture engine for XV',1)
		s.addi('"UsePIO"\t"false"\t# Use PIO instead of MMIO, (slower)',1)

### 3dfx card, Voodoo 1 or 2 with open driver
class Dev_voodoo(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "voodoo"
		self.dri = 1 ### FIXME Check this
		self.driver_description = "Voodoo 1/2"

### Maxtrox card with open driver
class Dev_mga(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "mgs"
		self.dri = 1 
		self.driver_description = "Matrox Mystique/Millenium/G100/G200/G400/G550"

	def unset_driver_options(self, s):
		s.addi('"MGASDRAM"\t"auto"\t# Forces SDRAM on/off (useful for secondary heads)',1)
		s.addi('"NoAccel"\t"false"\t# Disables ALL hardware acceleration',1)
		s.addi('"Overlay"\t"false"\t# Enables 8 + 24 overlay mode (24 bit only)',1)
		s.addi('"Rotate"\t"off"\t# Rotates the display', 1)
		s.addi('"ShadowFB"\t"false"\t# Use the ShadowFB (required for rotation)',1)
		s.addi('"TexturedVideo"\t\"false"\t# Use the texture engine for XV',1)
		s.addi('"UseFBDev"\t"false"\t# Try enabling this if it does not work',1)

### VESA device driver
class Dev_vesa(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "vesa"
		
	def unset_driver_options(self, s):
		s.addi('"ShadowFB"\t"false"\t# Use the ShadowFB (required for rotation)',1)
		s.addi('"ModeSetClearScreen"\t"true"\t#If you have problems, turn this off)',1)

### Intel device driver
class Dev_i740(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "i740"
		self.driver_description = "Intel i740"

### Intel device driver
class Dev_i810(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "i840"
		self.driver_description = "Intel i8xx/i9xx"

### NeoMagic device driver
class Dev_neomagic(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "neomagic"
		self.driver_description = "NeoMagic"

### S3


### Sun Cards

### Sun Creator series cards
class Dev_sunffb(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "sunffb"
		self.driver_description = "Sun Creator"

### Sun CG6 cards
class Dev_sunc6g(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "suncg6"
		self.driver_description = "Sun CG6"

### Sun Leo/ZX cards
class Dev_sunleo(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "sunleo"
		self.driver_description = "Sun Leo/ZX"
	
### Sun CG14 cards
class Dev_suncg14(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "suncg14"
		self.driver_description = "Sun CG14"

### Sun BW2 cards
class Dev_sunbw2(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "sunbw2"
		self.driver_description = "Sun BW2"

### Sun CG3 cards
class Dev_suncg3(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "suncg3"
		self.driver_description = "Sun CG3"

### Sun TCX cards
class Dev_suntcx(XAC_Device):
	def __init__(self):
		XAC_Device.__init__(self)
		self.driver = "suntcx"
		self.driver_description = "Sun TCX"

### FIXME Add the rest
