#!/usr/bin/python
from device import *
from array import array
from struct import unpack
from fcntl import ioctl
from monitor import Mode,Modeline

### Frame Buffer Screen Info
FBIOGET_FSCREENINFO = 0x4602
### Frame Buffer Screen Info
FBIOGET_VSCREENINFO = 0x4600
### Size of fb_fix_screeninfo
FB_FIX_SCREENINFO_SZ = 68
### Size of fb_var_screeninfo
FB_VAR_SCREENINFO_SZ = 160

### Flags
FB_VMODE_INTERLACED = 1
FB_VMODE_DOUBLE = 2
FB_SYNC_HOR_HIGH_ACT = 1
FB_SYNC_VERT_HIGH_ACT = 2
FB_SYNC_COMP_HIGH_ACT = 8
FB_SYNC_BROADCAST = 16

### Framebuffer class
class Framebuffer:
	### Uses the FB ioctl settings to get info on the active FB
	def __init__(self):
		self.fix = {}
		self.var = {}
		### This is the only way I've found to get the ioctl to return
		### the whole structure
		fix = array('B', [0] * FB_FIX_SCREENINFO_SZ) 
		var = array('B', [0] * FB_VAR_SCREENINFO_SZ)
		### Open the FB Device (must be done as root!)
		try:
			FB = open("/dev/fb0", "r")
			### FB ioctl (FBIOGET_FSCREENINFO)
			ioctl(FB, FBIOGET_FSCREENINFO, fix, 1)
			### FB ioctl (FBIOGET_VSCREENINFO)
			ioctl(FB, FBIOGET_VSCREENINFO, var, 1)
			self.valid = 1
		except:
			self.valid = 0

		### Note! The fix and var structs aren't packed
		### This means that the below unpacking might be wrong on
		### non-32bit aligned systems FIXME?

		### Unpack the "fix" struct
		( 
			self.fix['id'],
			self.fix['smem_start'],
			self.fix['smem_len'],
			self.fix['type'],
			self.fix['type_aux'],
			self.fix['visual'],
			self.fix['xpanstep'],
			self.fix['ypanstep'],
			self.fix['ywrapsetp'],
			self.fix['reserved'],
			self.fix['line_length'],
			self.fix['mmio_start'],
			self.fix['mmio_len'],
			self.fix['accel'],
			self.fix['reserved'],
			self.fix['reserved'],
			self.fix['reserved'],
			self.fix['reserved'],
		) = unpack("16s5I4H4I4H",fix)

		### Unpack the "var" struct
		(
			self.var['xres'],
			self.var['yres'],
			self.var['xres_virtual'],
			self.var['yres_virtual'],
			self.var['xoffset'],
			self.var['yoffset'],
			self.var['bits_per_pixel'],
			self.var['grayscale'],
			self.var['red_offset'],
			self.var['red_length'],
			self.var['red_msb_right'],
			self.var['green_offset'],
			self.var['green_length'],
			self.var['green_msb_right'],
			self.var['blue_offset'],
			self.var['blue_length'],
			self.var['blue_msb_right'],
			self.var['transp_offset'],
			self.var['transp_length'],
			self.var['transp_msb_right'],
			self.var['nonstd'],
			self.var['activate'],
			self.var['height'],
			self.var['width'],
			self.var['accel_flags'],
			self.var['pixclock'],
			self.var['left_margin'],
			self.var['right_margin'],
			self.var['upper_margin'],
			self.var['lower_margin'],
			self.var['hsync_len'],
			self.var['vsync_len'],
			self.var['sync'],
			self.var['vmode'],
			self.var['rotate'],
			self.var['reserved1'],
			self.var['reserved2'],
			self.var['reserved3'],
			self.var['reserved4'],
			self.var['reserved5'],
		) = unpack("40I",var)
	
	### Get the Identification String
	def fb_string(self):
		return self.fix['id'].strip('\000').strip()


	### Get the FB depth
	def depth(self):
		return self.var['bits_per_pixel']

### Commented out for now...
#		if self.var['bits_per_pixel'] == 8:
#			return 8
#		else:
#			return self.var['red_length'] + self.var['green_length'] + self.var['blue_length']

	### Get the FB Resolution (x,y)
	def res(self):
		return [self.var['xres'], self.var['yres']] 
	
	### Get the start of the video memory region
	def mem_start(self):
		return self.fix['smem_start']

	### This function is only called for non-pci cards
	def fb_id_to_compatible(self, binary_drivers):
		id = self.fix['accel']

		### PS3
		if id == 0 and self.fix['id'].count("PS3"):
			return [Dev_ps3()]
		else:
			return [Dev_fbdev()]
		
	### Convert the FB ID # to the correct driver
	### IDs taken from "linux/fb.h"
	def fb_id_to_driver(self, binary_drivers):
		if not self.valid:
			return None

		id = self.fix['accel']

		### PS3
		if id == 0 and self.fix['id'].count("PS3"):
			return Dev_ps3()	
		### Mach64
		elif [6, 8, 9, 10].count(id) > 0:
			return Dev_mach64()
		### Sun Creator 
		elif id == 11:
			return Dev_sunffb()
		### Sun CG6
		elif id == 12:
			return Dev_suncg6()
		### Sun Leo/ZX
		elif id == 13:
			return Dev_sunleo()
		### IMS Twin Turbo
		elif id == 14:
			print "FIXME: IMS Twin Turbo!\n"
			pass ### FIXME
		### Matrox
		elif [16,17,18, 19, 20, 21, 26].count(id) > 0:
			return Dev_mga()
		### Sun CG14
		elif id == 22:
			return Dev_suncg14()
		### Sun BW2
		elif id == 23:
			return Dev_sunbw2()
		### Sun CG3
		elif id == 24:
			return Dev_suncg3()
		### Sun TCX
		elif id == 25:
			return Dev_suntcx()
		### Chips & Technology 6555x
		elif id == 30:
			print "FIXME: Chips & Tech 6555x\n"
			pass ### FIXME
		### Voodoo3 / Banshee
		elif id == 31:
			return Dev_tdfx()
		### ATI Rage 128
		elif id == 32:
			return Dev_r128()
		### ATI Radeon
		elif id == 38:
			if binary_drivers:
				return Dev_radeonb()
			else:
				return Dev_radeon()
		### nVidia
		elif [27, 28, 29, 43, 44, 45, 46].count(id) > 0:
			if binary_drivers:
				return Dev_nvidiab()
			else:
				return Dev_nvidia()
		### Intel
		elif [39, 42].count(id) > 0:
			return Dev_i810()
		### NeoMagic
		elif [90, 91, 92, 93, 94, 95, 96, 97, 98].count(id) > 0:
			return Dev_neomagic()
		### Give up
		else:
			return None
	
	### Get the flags we add to the end of the modeline
	def get_ml_flags(self):
		flags = ""
		if self.var['vmode'] & FB_VMODE_INTERLACED:
			flags = flags + " Interlace"
		if self.var['vmode'] & FB_VMODE_DOUBLE:
			flags = flags + " DoubleScan"

		if self.var['sync'] & FB_SYNC_HOR_HIGH_ACT:
			flags = flags + " +HSync"
		else:
			flags = flags + " -HSync"

		if self.var['sync'] & FB_SYNC_VERT_HIGH_ACT:
			flags = flags + " +VSync"
		else:
			flags = flags + " -VSync"
		if self.var['sync'] & FB_SYNC_COMP_HIGH_ACT:
			flags = flags + " Composite"
		if self.var['sync'] & FB_SYNC_BROADCAST:
			flags = flags + " bcast"

		return flags

	### Get a mode for the running FB
	def get_mode(self):
		var = self.var
		htotal = var['left_margin'] + var['xres'] + var['right_margin'] + var['hsync_len']
		vtotal = var['upper_margin'] + var['yres'] + var['lower_margin'] + var['vsync_len']
		drate = 1000000000000.0 / var['pixclock']

		hrate = drate / htotal
		vrate = hrate / vtotal

		xs_start = var['xres'] + var['right_margin']
		xs_end = xs_start + var['hsync_len']
		xtotal = xs_end + var['left_margin']

		ys_start = var['yres'] + var['lower_margin']
		ys_end = ys_start + var['vsync_len']
		ytotal = ys_end + var['upper_margin']

		comment = "Framebuffer\tD: " + str(drate / 1000000.0)[:8] + " MHz, H: " + str(hrate / 1000.0)[:8] +" kHz, V: " + str(vrate)[:8] + " Hz" 
		line = 'Modeline\t"%dx%d" %5.3f %d %d %d %d\t%d %d %d %d' % (var['xres'], var['yres'], drate / 1000000.0, var['xres'], xs_start, xs_end, xtotal, var['yres'], ys_start, ys_end, ytotal)

		line = line + self.get_ml_flags() 

		### Reduced blanking check
		if xtotal - var['xres'] == 160:
			rb = 1
		else:
			rb = 0

		return Mode(var['xres'], var['yres'], self.depth, Modeline(var['xres'], var['yres'], comment, line, rb))
